-----------------------------------------------------------------------------------------------
-- Client Lua Script for GuildCount
-- Copyright (c) NCsoft. All rights reserved
-- Author: Jerodar
-- Creates a Interface menu item that shows the online guildmember count
-- and can be clicked to open the guild menu
-----------------------------------------------------------------------------------------------
-- V1.0 - 	Found a nice sprite of a shield to use, added a ton of 
-- 			comments to explain the new API used.
-- V0.9 - 	Initial release, still uses the same icon as social
-----------------------------------------------------------------------------------------------

require "Window"
 
local GuildCount = {} 
 
-- KEY
local tKey = {}
		tKey['a'] = 65
		tKey['b'] = 66
		tKey['c'] = 67
		tKey['d'] = 68
		tKey['e'] = 69
		tKey['f'] = 70
		tKey['g'] = 71
		tKey['h'] = 72
		tKey['i'] = 73
		tKey['j'] = 74
		tKey['k'] = 75
		tKey['l'] = 76
		tKey['m'] = 77
		tKey['n'] = 78
		tKey['o'] = 79
		tKey['p'] = 80
		tKey['q'] = 81
		tKey['r'] = 82
		tKey['s'] = 83
		tKey['t'] = 84
		tKey['u'] = 85
		tKey['v'] = 86
		tKey['w'] = 87
		tKey['x'] = 88
		tKey['y'] = 89
		tKey['z'] = 90
		tKey['0'] = 96
		tKey['1'] = 97
		tKey['2'] = 98
		tKey['3'] = 99
		tKey['4'] = 100
		tKey['5'] = 101
		tKey['6'] = 102
		tKey['7'] = 103
		tKey['8'] = 104
		tKey['9'] = 105

-----------------------------------------------------------------------------------------------
-- Initialization
----------------------------------------------------------------------------------------------- 
 
function GuildCount:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	
    o.nKeybind=nil

	o.tRanks = {}

    return o
end

function GuildCount:Init()
    Apollo.RegisterAddon(self)
end
 
function GuildCount:OnLoad()
	-- registers the slash command
	Apollo.RegisterSlashCommand("gc", "OnSlashCommand", self)

    -- Triggers when Interface menu list is loaded
	-- You can then register new entries for the list
	Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
	
	-- Triggers when a guild member comes online/offline or joins/leaves the guild.
	-- The addon will then update the membercount
	Apollo.RegisterEventHandler("GuildMemberChange", "OnGuildMemberChange", self)
	-- This event triggers after a RequestMembers() call and supplies the roster to the addon
	Apollo.RegisterEventHandler("GuildRoster", "OnGuildRoster", self)

	-- Fires when the user presses any key
	Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self) 
end

function GuildCount:OnSave(eLevel)
	-- General = 1 save across all accounts, realms and character
    if eLevel ~= GameLib.CodeEnumAddonSaveLevel.General then
        return nil
    end

    local tData = {}
    tData.nKeybind = self.nKeybind

    return tData
end

function GuildCount:OnRestore(eLevel, tData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.General then
        return nil
    end

    if not tData then
    	Print("ERROR: No save data")
    end

	self.nKeybind = tData.nKeybind
end

-----------------------------------------------------------------------------------------------
-- GuildCount functions
----------------------------------------------------------------------------------------------- 

function GuildCount:OnSlashCommand(strCmd, strArgs)
	--split up the args into single words
	local tArgs = {}
	for i in strArgs:gmatch("%S+") do table.insert(tArgs, i) end
	if tArgs[1] then
		self.nKeybind = tKey[tArgs[1]]
		Print("GuildCount: Keybind set to '".. tArgs[1] .. "'.")
	else
		self.nKeybind = nil
		Print("GuildCount: Keybind is now unbound.")
	end
end

function GuildCount:OnInterfaceMenuListHasLoaded()
	-- Register a new menu entry
	-- 1st arg: string that is displayed in the menu, also used to id the menuitem
	-- 2nd arg: an array with 3 fields
		-- 1st field: The event that is triggered when the item is clicked
		-- 2nd field: The keybind it displays (same name as in keybind menu)
		-- 3rd field: The sprite that is displayed, "" will display first letter of menu item
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Guild", {"GenericEvent_OpenGuildPanel", "Guild", "IconSprites:Icon_ArchetypeUI_CRB_Guard"})
	 
	 -- possible alternative icons
	-- IconSprites:Icon_Mission_Explorer_ClaimTerritory
	-- IconSprites:Icon_Mission_Settler_Scout
	-- IconSprites:Icon_MapNode_Map_PvP_ExileCarry
	-- IconSprites:Icon_Windows_UI_CRB_Attribute_BruteForce

	-- Get online member count
	local nGuildMemberCount = 0
	for key, tGuild in pairs(GuildLib.GetGuilds()) do
		if tGuild:GetType() == GuildLib.GuildType_Guild then
			-- And request a roster, it will arrive through the GuildRoster event
			tGuild:RequestMembers()
		end
	end
end

-- Called whenever a guild member comes online/offline or enters/leaves the guild
function GuildCount:OnGuildMemberChange(guildUpdated)
	-- if it's a circle and not a guild stop
	if guildUpdated:GetType() ~= GuildLib.GuildType_Guild then return end

	-- Request a roster, it will arrive through the GuildRoster event
	guildUpdated:RequestMembers()
end

-- 
function GuildCount:OnGuildRoster(guildCurr, tRoster)
	if guildCurr:GetType() ~= GuildLib.GuildType_Guild then return end

	-- Get the online member count to display on the icon
	local nOnlineCount = guildCurr:GetOnlineMemberCount()

	-- Count the members in each rank
	for idx, tRank in pairs(guildCurr:GetRanks()) do
		self.tRanks[idx] = {}
		self.tRanks[idx].strName = tRank.strName
		self.tRanks[idx].nCount = 0
		self.tRanks[idx].nOnlineCount = 0
	end

	for idx, tMember in pairs(tRoster) do
		self.tRanks[tMember.nRank].nCount = self.tRanks[tMember.nRank].nCount + 1
		if tMember.fLastOnline == 0 then
			self.tRanks[tMember.nRank].nOnlineCount = self.tRanks[tMember.nRank].nOnlineCount + 1
		end
	end

	-- Get the total amount of guildmembers
	local nTotalCount = guildCurr:GetMemberCount()

	-- Build the tooltip
	local sTooltip = " \n" .. guildCurr:GetName() .. ": " ..  nOnlineCount .. "/" .. nTotalCount .. "\n \n"
	for idx, tRank in pairs(self.tRanks) do
		if tRank.strName ~= "" then
			sTooltip = sTooltip .. tRank.strName .. ": " .. tRank.nOnlineCount .. "/" .. tRank.nCount .. "\n"
		end
	end

	-- Updates the icon
	-- 1s arg: The same name as used when registering the menu item
	-- 2nd arg: an array with 3 fields
		-- 1st field: Highlight icon true/false
		-- String that is concatenated to the tooltip
		-- Integer that is displayed on top of the icon if not 
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "Guild", {false, sTooltip, nOnlineCount})
end

function GuildCount:OnSystemKeyDown(iKey)
	if self.nKeybind and iKey == self.nKeybind then
		Event_FireGenericEvent("GenericEvent_OpenGuildPanel");
	end
end

local GuildCountInst = GuildCount:new()
GuildCountInst:Init()